@@ -396,8 +396,18 @@ async def submit_prompt(request: SubmitPromptRequest):
                     result = await _run_sequential_chain(runner, request.agents, request.inputs, pid)
                 else:
                     # Single agent mode (default)
+                    # For single-mode prompt executions initiated from the API UI,
+                    # treat them as user-facing one-shot runs where the user likely
+                    # does not expect the agent to autonomously delegate to other
+                    # agents (e.g., researcher delegating to coordinator). We set
+                    # an explicit override flag in inputs to discourage delegation
+                    # unless the caller explicitly enables it.
+                    inputs_for_run = dict(request.inputs or {})
+                    # Use a private key to avoid colliding with user inputs
+                    inputs_for_run["_allow_delegation"] = False
+
                     result = await runner.run(
-                        inputs=request.inputs,
+                        inputs=inputs_for_run,
                         agent_name=request.prompt_name,
                         job_id=pid,
                     )
@@ -884,7 +894,10 @@ async def websocket_prompt_traces(websocket: WebSocket, prompt_id: str):
     await websocket.accept()
     logger.info(f"WebSocket connected for prompt {prompt_id}")
     
-    # Track last seen trace ID to avoid duplicates
+    # Track last seen trace timestamp to avoid duplicates (use timestamp-based filtering to
+    # avoid races where autoincrement id ordering differs from timestamps). Fall back to
+    # last_trace_id when timestamps are missing.
+    last_seen_ts: datetime | None = None
     last_trace_id = 0
     # Track open delegation windows to annotate child traces with parent_trace_id
     # Shape: { parent_trace_id: { 'agent': str|None, 'start': datetime, 'end': datetime|None } }
@@ -904,12 +917,33 @@ async def websocket_prompt_traces(websocket: WebSocket, prompt_id: str):
             # Get traces for this prompt/job
             traces = database.get_job_traces(prompt_id)
             
-            # Filter to only new traces
-            new_traces = [t for t in traces if t.get('id', 0) > last_trace_id]
-            
+            # Filter to only new traces using timestamp-based comparison when possible.
+            parsed_traces = []
+            for t in traces:
+                ts = _parse_ts(t.get('timestamp'))
+                parsed_traces.append((t, ts))
+
+            if last_seen_ts is None:
+                # First time: send all traces
+                new_traces = [t for t, _ in parsed_traces]
+            else:
+                # Send traces whose timestamp is greater than last_seen_ts
+                new_traces = [t for t, ts in parsed_traces if ts is not None and ts > last_seen_ts]
+
+            # Fallback: if no timestamps available or no new traces found, use id-based filtering
+            if not new_traces:
+                new_traces = [t for t in traces if t.get('id', 0) > last_trace_id]
+
             if new_traces:
-                # Update last seen
-                last_trace_id = max(t.get('id', 0) for t in new_traces)
+                # Update last seen timestamp and id conservatively
+                max_ts = None
+                for t in new_traces:
+                    ts = _parse_ts(t.get('timestamp'))
+                    if ts and (max_ts is None or ts > max_ts):
+                        max_ts = ts
+                if max_ts:
+                    last_seen_ts = max_ts
+                last_trace_id = max(last_trace_id, max(t.get('id', 0) for t in new_traces))
                 
                 # Send new traces
                 for trace in new_traces:
@@ -919,14 +953,21 @@ async def websocket_prompt_traces(websocket: WebSocket, prompt_id: str):
                         payload = trace.get('payload') or {}
                         tool_name = (payload.get('tool') or '').lower()
 
-                        # Delegation start (both single and parallel)
-                        if event_type == 'tool_call' and tool_name in {
+                        # Delegation start (detect both system tool-based delegation and custom delegation events)
+                        if (event_type == 'tool_call' and tool_name in {
                             'delegate_parallel', 'system_delegate_parallel',
                             'delegate_task', 'system_delegate_task'
-                        }:
+                        }) or event_type == 'delegation':
                             parent_id = trace.get('id')
                             params = payload.get('params') or {}
-                            delegated_agent = params.get('agent') or params.get('agent_name')
+                            # For custom delegation traces the payload may use different keys
+                            delegated_agent = (
+                                params.get('agent')
+                                or params.get('agent_name')
+                                or payload.get('target_agent')
+                                or payload.get('delegated_agent')
+                                or payload.get('agent')
+                            )
                             open_delegations[int(parent_id)] = {
                                 'agent': delegated_agent,
                                 'start': _parse_ts(trace.get('timestamp')),
