@@ -88,7 +88,6 @@ def add_agent(
 
     print_success(f"Agent {agent_name} created")
     print_info(f"  Agent: agents/{agent_name}.py")
-    print_info(f"  Worker: workers/{agent_name}_worker.py")
     print_info("  Docker service added to docker-compose.yml")
 
 
@@ -174,10 +173,9 @@ def _create_agent_files(
         "available_agents": [],
         "model_env": f"{agent_name.upper()}_MODEL",
         "model": llm_model,
-        "fallback_model_env": f"{agent_name.upper()}_FALLBACK_MODEL",
-        "fallback_model": "gpt-4o-mini",
         "temperature": 0.3 if agent_name.lower() == "coordinator" else 0.2,
         "max_retries": 3,
+        "max_iterations": 5 if agent_name.lower() == "coordinator" else 15,
         "timeout": 600,
         "instructions": (
             "When you receive a task: 1) Analyze 2) Plan 3) Execute using tools 4) Synthesize results."
@@ -189,7 +187,6 @@ def _create_agent_files(
 
     # Worker entry using WorkerRunner template
     worker_context = {"agent_name": agent_name}
-    renderer.render_to_file("worker_runner.py.j2", Path("workers") / f"{agent_name}_worker.py", worker_context)
 
     # Update evenage.yml
     with open("evenage.yml", "r", encoding="utf-8") as f:
@@ -204,9 +201,6 @@ def _create_agent_files(
     # Add worker service to docker-compose.yml
     add_worker_to_compose(Path.cwd(), agent_name)
 
-    # Auto-register agent on message bus (if Redis is available)
-    _auto_register_agent(agent_name, role, goal, backstory)
-
 
 def _create_tool_file(tool_name: str, agent_name: str, description: str) -> None:
     """Create tool file (project-level) and update agent configuration."""
@@ -225,40 +219,3 @@ def _create_tool_file(tool_name: str, agent_name: str, description: str) -> None
     }
     renderer.render_to_file("tool.py.j2", tools_dir / f"{tool_name}.py", tool_context)
 
-
-def _auto_register_agent(agent_name: str, role: str, goal: str, backstory: str | None) -> None:
-    """Auto-register agent on message bus if Redis is available."""
-    try:
-        import asyncio
-
-        from evenage.core import BackendFactory, EvenAgeConfig
-
-        # Try to connect to Redis
-        config = EvenAgeConfig()
-        factory = BackendFactory(config)
-
-        async def register():
-            bus = factory.create_queue_backend()
-            database = factory.create_database_backend()
-
-            metadata = {
-                "role": role,
-                "goal": goal,
-                "backstory": backstory or "",
-                "status": "inactive",
-                "tools": [],
-                "allow_delegation": False,
-                "max_iterations": 15
-            }
-
-            # Register on bus and database
-            await bus.register_agent(agent_name, metadata)
-            database.register_agent(agent_name, metadata)
-            print_info("  Agent registered on message bus")
-
-        asyncio.run(register())
-
-    except Exception as e:
-        # Redis not available or other error - not critical
-        print_info(f"  Skipping auto-registration (Redis not available): {e}")
-
