@@ -3,7 +3,7 @@ EvenAge Core module.
 
 Exports:
 - Configuration: EvenAgeConfig, AgentConfig, ProjectConfig
-- Agent runtime: Agent, AgentRunner, run_agent, PausedExecutionContext
+- Agent runtime: Agent, AgentRunner, run_agent
 - Decorators: tool
 - Backends: BackendFactory
 - Database: DatabaseService
@@ -13,7 +13,7 @@ Exports:
 # NOTE: MCP feature disabled for this release
 # - MCP: MCPToolSource, MCPToolRegistry
 
-from .agent_runtime import Agent, AgentMemory, PausedExecutionContext
+from .agent_runtime import Agent, AgentMemory
 from .cache import InMemoryCache, RedisCache
 from .config import (
     AgentConfig,
@@ -58,7 +58,6 @@ __all__ = [
     # Core classes
     "Agent",
     "AgentMemory",
-    "PausedExecutionContext",
     "AgentRunner",
     "WorkerRunner",
     "run_agent",
