@@ -45,77 +45,6 @@ class AgentMemory:
         return self.database.memory_list(self.agent_name, self.job_id)
 
 
-@dataclass
-class PausedExecutionContext:
-    """
-    Saved execution context for paused agents waiting on delegation.
-    
-    This enables non-blocking delegation: when an agent delegates a task,
-    it saves its state and returns control to the orchestrator. When the
-    delegated task completes, the orchestrator resumes the parent agent
-    from this saved context.
-    """
-    parent_task_id: str  # The task_id of the parent agent's execution
-    parent_agent_name: str  # Name of the agent that delegated
-    delegation_task_id: str  # Task_id of the delegated subtask
-    delegated_agent_name: str  # Target agent for delegation
-    delegated_task: dict  # The task payload sent to delegated agent
-    
-    # Current execution state
-    task: dict  # Original task inputs
-    history: list[dict]  # Conversation history up to delegation point
-    iteration: int  # Which iteration was paused
-    max_iterations: int  # Max iterations allowed
-    max_tool_calls: int | None  # Max tool calls allowed (None = unlimited)
-    
-    # Deduplication tracking
-    seen_queries: set[str]  # Query deduplication cache
-    seen_delegations: dict[str, set[str]]  # Delegation deduplication cache
-    
-    # Timestamps
-    paused_at: str  # ISO timestamp when paused
-    timeout_seconds: int  # How long to wait for delegation response
-    
-    def to_dict(self) -> dict:
-        """Serialize to dict for storage."""
-        return {
-            "parent_task_id": self.parent_task_id,
-            "parent_agent_name": self.parent_agent_name,
-            "delegation_task_id": self.delegation_task_id,
-            "delegated_agent_name": self.delegated_agent_name,
-            "delegated_task": self.delegated_task,
-            "task": self.task,
-            "history": self.history,
-            "iteration": self.iteration,
-            "max_iterations": self.max_iterations,
-            "max_tool_calls": self.max_tool_calls,
-            "seen_queries": list(self.seen_queries),
-            "seen_delegations": {k: list(v) for k, v in self.seen_delegations.items()},
-            "paused_at": self.paused_at,
-            "timeout_seconds": self.timeout_seconds,
-        }
-    
-    @classmethod
-    def from_dict(cls, data: dict) -> "PausedExecutionContext":
-        """Deserialize from dict."""
-        return cls(
-            parent_task_id=data["parent_task_id"],
-            parent_agent_name=data["parent_agent_name"],
-            delegation_task_id=data["delegation_task_id"],
-            delegated_agent_name=data["delegated_agent_name"],
-            delegated_task=data["delegated_task"],
-            task=data["task"],
-            history=data["history"],
-            iteration=data["iteration"],
-            max_iterations=data["max_iterations"],
-            max_tool_calls=data.get("max_tool_calls"),
-            seen_queries=set(data.get("seen_queries", [])),
-            seen_delegations={k: set(v) for k, v in data.get("seen_delegations", {}).items()},
-            paused_at=data["paused_at"],
-            timeout_seconds=data.get("timeout_seconds", 60),
-        )
-
-
 class Agent:
     """
     Base Agent class with full runtime capabilities.
@@ -123,22 +52,11 @@ class Agent:
     Provides:
     - Planning and execution
     - Tool calling with validation and caching
-    - Non-blocking delegation with pause/resume orchestration
+    - Delegation with blocking wait for responses
     - Memory storage
     - Auto-registration on message bus
     - Trace logging to database
     
-    Delegation Model:
-    - Modern autonomous agents use NON-BLOCKING delegation
-    - When delegating, the agent saves its state and returns status="paused"
-    - The orchestrator calls resume_paused_execution() when delegation completes
-    - This enables event-driven, scalable multi-agent workflows
-    
-    Backward Compatibility:
-    - Custom plan() methods can still use blocking delegation via wait_for_response
-    - System tools like delegate_task support wait_for_response parameter
-    - All existing agent code continues to work unchanged
-    
     Subclasses should override plan() method for custom planning logic,
     or use the default autonomous_run() for intelligent LLM-driven execution.
     """
@@ -463,6 +381,9 @@ class Agent:
                 # Extract execution limits from task (if provided by user)
                 max_iterations = task.get("max_iterations")
                 max_tool_calls = task.get("max_tool_calls")
+                # Fall back to instance-level max_tool_calls if set
+                if max_tool_calls is None and hasattr(self, "_max_tool_calls"):
+                    max_tool_calls = self._max_tool_calls
                 result = await self.autonomous_run(
                     task, 
                     max_iterations=max_iterations,
@@ -481,22 +402,65 @@ class Agent:
                 
                 result = await self.execute_plan(plan, task)
 
-            # Trace completion
+            # Trace completion / final state
             task_ended_at = _dt.utcnow().isoformat() + "Z"
-            self._trace(
-                execution_id,
-                self.config.name,
-                "task_complete",
-                {
-                    "result": result,
-                    "worker": self.config.name,
-                    "ended_at": task_ended_at
-                }
-            )
+            try:
+                self._trace(
+                    execution_id,
+                    self.config.name,
+                    "task_complete",
+                    {
+                        "result": result,
+                        "worker": self.config.name,
+                        "ended_at": task_ended_at
+                    }
+                )
+            except Exception:
+                pass
+
+            # If the agent returned a structured status, propagate it instead of
+            # always returning "success". This preserves error/incomplete
+            # semantics from autonomous_run() so the API can handle failures correctly.
+            if isinstance(result, dict) and result.get("status"):
+                res_status = result.get("status")
+
+                # Auto-route specialist results to coordinator only on final success
+                if res_status == "success" and not self.is_coordinator and self.coordinator_agent:
+                    try:
+                        await self.bus.publish_task(
+                            self.coordinator_agent,
+                            {
+                                "type": "specialist_result",
+                                "source_agent": self.config.name,
+                                "job_id": execution_id,
+                                "result": result
+                            }
+                        )
+                    except Exception:
+                        pass
+
+                # Map statuses through
+                if res_status == "error":
+                    return {
+                        "status": "error",
+                        "error": result.get("error") or "error",
+                        "result": result,
+                        "job_id": execution_id,
+                        "prompt_id": execution_id
+                    }
+                if res_status in {"incomplete", "canceled"}:
+                    return {
+                        "status": res_status,
+                        "result": result,
+                        "job_id": execution_id,
+                        "prompt_id": execution_id
+                    }
+
+            # Default: treat as successful completion and return extracted payload
+            outputs = result.get("result") if isinstance(result, dict) else result
 
             # Auto-route result to coordinator if applicable
             if not self.is_coordinator and self.coordinator_agent:
-                # Send result back to coordinator for synthesis
                 try:
                     await self.bus.publish_task(
                         self.coordinator_agent,
@@ -504,18 +468,17 @@ class Agent:
                             "type": "specialist_result",
                             "source_agent": self.config.name,
                             "job_id": execution_id,
-                            "result": result
+                            "result": outputs
                         }
                     )
                 except Exception:
-                    # Non-fatal: continue even if coordinator routing fails
                     pass
 
             return {
                 "status": "success",
-                "result": result.get("result") if isinstance(result, dict) else result,
-                "job_id": execution_id,  # Keep for backward compatibility (no clue why this is here maybe we'll find out one day)
-                "prompt_id": execution_id  # Add new terminology
+                "result": outputs,
+                "job_id": execution_id,
+                "prompt_id": execution_id,
             }
 
         except Exception as e:
@@ -1150,60 +1113,48 @@ class Agent:
                 # Record this delegation
                 seen_delegations[agent_name].add(normalized_task)
 
-                # NON-BLOCKING DELEGATION: Save context and pause execution
+                # Simple blocking delegation
                 try:
-                    # Publish delegation task
-                    delegation_task_payload = {"query": delegate_task}
-                    delegation_task_id = await self.delegate(agent_name, delegation_task_payload)
-                    logger.info(
-                        "[%s] delegation task_id=%s to %s (NON-BLOCKING); pausing execution...",
-                        self.config.name,
-                        delegation_task_id,
-                        agent_name
-                    )
-                    
-                    # Create paused context
-                    from datetime import datetime as _dt
-                    paused_at = _dt.utcnow().isoformat() + "Z"
-                    timeout = action.get("timeout", 60)
-                    
-                    paused_context = PausedExecutionContext(
-                        parent_task_id=self.current_job_id,
-                        parent_agent_name=self.config.name,
-                        delegation_task_id=delegation_task_id,
-                        delegated_agent_name=agent_name,
-                        delegated_task=delegation_task_payload,
-                        task=task,
-                        history=history,
-                        iteration=iteration,
-                        max_iterations=max_iter,
-                        max_tool_calls=max_tools,
-                        seen_queries=seen_queries,
-                        seen_delegations=seen_delegations,
-                        paused_at=paused_at,
-                        timeout_seconds=timeout
-                    )
-                    
-                    # Save paused context
-                    await self._save_paused_context(paused_context)
-                    
-                    # Return PAUSED status to orchestrator
-                    logger.info(
-                        "[%s] Execution PAUSED at iteration %d, waiting for delegation_task_id=%s",
-                        self.config.name,
-                        iteration,
-                        delegation_task_id
-                    )
-                    return {
-                        "status": "paused",
-                        "message": f"Execution paused, waiting for delegation to {agent_name}",
-                        "delegation_task_id": delegation_task_id,
-                        "delegated_agent": agent_name,
-                        "parent_task_id": self.current_job_id,
-                        "iteration": iteration,
-                        "prompt_id": self.current_job_id
+                    delegation_task_payload = {
+                        "query": delegate_task,
+                        "job_id": self.current_job_id,
                     }
-                    
+
+                    # Delegate (delegate() will also ensure job_id propagation if missing)
+                    delegation_task_id = await self.delegate(agent_name, delegation_task_payload)
+
+                    # Always block and wait for response
+                    timeout = action.get("timeout", 120)
+                    logger.info("[%s] delegating task_id=%s to %s, waiting for response (timeout=%ss)...", 
+                               self.config.name, delegation_task_id, agent_name, timeout)
+                    response = None
+                    try:
+                        response = await self.bus.wait_for_response(delegation_task_id, timeout)
+                    except Exception as e:
+                        logger.warning("[%s] error while waiting for delegation response: %s", self.config.name, e)
+
+                    if response is None:
+                        # No response within timeout
+                        logger.warning("[%s] delegation %s timed out after %ss", self.config.name, delegation_task_id, timeout)
+                        history.append({
+                            "action": "delegate",
+                            "agent": agent_name,
+                            "task": delegate_task,
+                            "error": f"Delegation timed out after {timeout}s",
+                            "task_id": delegation_task_id
+                        })
+                    else:
+                        # Got response: inject into history and continue loop
+                        history.append({
+                            "action": "delegate",
+                            "agent": agent_name,
+                            "task": delegate_task,
+                            "result": response,
+                            "status": response.get("status") if isinstance(response, dict) else "unknown",
+                        })
+                        logger.info("[%s] delegation %s returned with status=%s", 
+                                   self.config.name, delegation_task_id, history[-1].get("status"))
+
                 except Exception as e:
                     logger.exception("[%s] delegation error: %s", self.config.name, e)
                     history.append({
@@ -1273,192 +1224,6 @@ class Agent:
             # Tracing must never break runtime
             pass
     
-    async def _save_paused_context(self, context: PausedExecutionContext) -> None:
-        """
-        Save paused execution context to storage for later resumption.
-        
-        Uses cache backend for fast access with TTL-based cleanup.
-        Stores in memory table as backup for durability.
-        """
-        cache_key = f"paused_context:{context.delegation_task_id}"
-        context_data = context.to_dict()
-        
-        # Store in cache with TTL (2x timeout for safety margin)
-        ttl = context.timeout_seconds * 2
-        try:
-            await self._put_in_cache(cache_key, context_data, ttl=ttl)
-        except Exception as e:
-            logger.warning("Failed to save paused context to cache: %s", e)
-        
-        # Store in database memory table as durable backup
-        try:
-            self.memory.put(
-                f"paused_context:{context.delegation_task_id}",
-                context_data
-            )
-        except Exception as e:
-            logger.warning("Failed to save paused context to database: %s", e)
-        
-        # Trace the pause event
-        self._trace(
-            context.parent_task_id,
-            context.parent_agent_name,
-            "execution_paused",
-            {
-                "delegation_task_id": context.delegation_task_id,
-                "delegated_agent": context.delegated_agent_name,
-                "iteration": context.iteration,
-                "paused_at": context.paused_at,
-            }
-        )
-    
-    async def _load_paused_context(self, delegation_task_id: str) -> PausedExecutionContext | None:
-        """
-        Load paused execution context from storage.
-        
-        Tries cache first (fast), falls back to database if needed.
-        """
-        cache_key = f"paused_context:{delegation_task_id}"
-        
-        # Try cache first
-        try:
-            context_data = await self._get_from_cache(cache_key)
-            if context_data:
-                return PausedExecutionContext.from_dict(context_data)
-        except Exception as e:
-            logger.warning("Failed to load paused context from cache: %s", e)
-        
-        # Fall back to database
-        try:
-            context_data = self.memory.get(f"paused_context:{delegation_task_id}")
-            if context_data:
-                return PausedExecutionContext.from_dict(context_data)
-        except Exception as e:
-            logger.warning("Failed to load paused context from database: %s", e)
-        
-        return None
-    
-    async def _cleanup_paused_context(self, delegation_task_id: str) -> None:
-        """Clean up paused context after successful resumption."""
-        cache_key = f"paused_context:{delegation_task_id}"
-        
-        # Remove from cache
-        try:
-            if hasattr(self.cache_backend, 'delete'):
-                if asyncio.iscoroutinefunction(self.cache_backend.delete):
-                    await self.cache_backend.delete(cache_key)
-                else:
-                    self.cache_backend.delete(cache_key)
-        except Exception:
-            pass
-        
-        # Remove from database
-        try:
-            # Memory table doesn't have explicit delete, but we can set to None
-            self.memory.put(f"paused_context:{delegation_task_id}", None)
-        except Exception:
-            pass
-    
-    async def resume_paused_execution(
-        self,
-        delegation_task_id: str,
-        delegation_result: dict
-    ) -> dict:
-        """
-        Resume a paused agent execution after delegation completes.
-        
-        This is called by the orchestrator when a delegated task finishes.
-        It reloads the parent agent's saved context and continues execution
-        from where it left off, injecting the delegation result into history.
-        
-        Args:
-            delegation_task_id: The task_id of the completed delegation
-            delegation_result: The result returned by the delegated agent
-        
-        Returns:
-            Final execution result (may be paused again if another delegation occurs)
-        """
-        # Load paused context
-        context = await self._load_paused_context(delegation_task_id)
-        if not context:
-            logger.error(
-                "Cannot resume execution for delegation_task_id=%s: context not found",
-                delegation_task_id
-            )
-            return {
-                "status": "error",
-                "error": f"Paused context not found for delegation {delegation_task_id}",
-                "prompt_id": self.current_job_id
-            }
-        
-        # Trace resumption
-        from datetime import datetime as _dt
-        resumed_at = _dt.utcnow().isoformat() + "Z"
-        self._trace(
-            context.parent_task_id,
-            context.parent_agent_name,
-            "execution_resumed",
-            {
-                "delegation_task_id": delegation_task_id,
-                "delegated_agent": context.delegated_agent_name,
-                "resumed_at": resumed_at,
-                "delegation_result_status": delegation_result.get("status"),
-            }
-        )
-        
-        # Restore execution state
-        task = context.task
-        history = context.history
-        iteration = context.iteration
-        max_iterations = context.max_iterations
-        max_tool_calls = context.max_tool_calls
-        seen_queries = context.seen_queries
-        seen_delegations = context.seen_delegations
-        
-        # Inject delegation result into history
-        result_status = delegation_result.get("status")
-        result_data = delegation_result.get("result") or delegation_result.get("error")
-        
-        history.append({
-            "action": "delegate",
-            "agent": context.delegated_agent_name,
-            "task": context.delegated_task.get("query", ""),
-            "result": delegation_result,
-            "status": result_status
-        })
-        
-        # Check if delegation failed - stop to avoid retry loops
-        if result_status in ["error", "incomplete"]:
-            logger.warning(
-                "[%s] Delegated task to %s returned status=%s, stopping to avoid retry loop",
-                self.config.name,
-                context.delegated_agent_name,
-                result_status
-            )
-            await self._cleanup_paused_context(delegation_task_id)
-            return {
-                "status": "error",
-                "error": f"Delegated task to {context.delegated_agent_name} failed: {result_data}",
-                "iterations": iteration + 1,
-                "history": history,
-                "prompt_id": context.parent_task_id
-            }
-        
-        # Clean up paused context (we're resuming now)
-        await self._cleanup_paused_context(delegation_task_id)
-        
-        # Continue autonomous execution from next iteration
-        # We call the internal loop helper to avoid re-setup
-        return await self._autonomous_run_loop(
-            task=task,
-            history=history,
-            start_iteration=iteration + 1,  # Continue from next iteration
-            max_iterations=max_iterations,
-            max_tool_calls=max_tool_calls,
-            seen_queries=seen_queries,
-            seen_delegations=seen_delegations
-        )
-    
     def _build_autonomous_system_prompt(
         self,
         available_tools: list[str],
@@ -1495,7 +1260,13 @@ class Agent:
 
         system = f"""{header}
 
-You act autonomously to complete the task. Always respond with a SINGLE JSON object only (no prose, no code fences).
+You act autonomously to complete the task. Always respond with a SINGLE JSON object only.
+
+CRITICAL: 
+- Output ONLY the JSON object, no markdown code fences (no ```json), no explanatory text
+- Do NOT add extra fields like "action", "reasoning", or "explanation"
+- Follow the EXACT schema below
+
 Schema:
 {{
   "type": "tool|delegate|finish",
@@ -1551,6 +1322,13 @@ To delegate a task:
 {{"type":"tool","tool":"system_get_artifact","params":{{"key":"<unique_id>"}}}}
 - Retrieve previously stored artifacts
 - Useful when building on work from previous steps or other agents
+ - IMPORTANT: Do NOT attempt to retrieve artifacts using job_id or task_id. Only call
+     `system_get_artifact` when you have an explicit artifact identifier (artifact_id)
+     or a direct bucket+key returned by a previous tool. If a delegated agent returned
+     its result inline (i.e., the response field contains the data), you do NOT need
+     to call artifact retrieval — synthesize from the inline result instead.
+     Calling artifact retrieval with job/task ids often fails (wrong bucket) and
+     causes unnecessary retries.
 """
 
         system += """
@@ -1606,9 +1384,23 @@ Think step-by-step internally, but output ONLY the JSON action object. No explan
                     prompt += f"\n⚠️ SYSTEM: {entry.get('message', '')}\n\n"
             
             prompt += "\n"
-        
+
+        # If a delegation result is present in history, add a short SYSTEM hint
+        # so the LLM prefers synthesizing and finishing over re-delegating the
+        # same task. This reduces redundant delegations and wasted tool calls.
+        has_delegate_result = False
+        for entry in history:
+            if entry.get("action") == "delegate" and "result" in entry and entry.get("status") in {"completed", "success", None}:
+                has_delegate_result = True
+                break
+
+        if has_delegate_result:
+            prompt += "\nSYSTEM_HINT: A delegated agent has returned a result above. "
+            prompt += "Prefer synthesizing a final answer from that result and other history. "
+            prompt += "Do NOT re-delegate the same task unless the returned result is clearly insufficient or explicitly requests further delegation.\n\n"
+
         prompt += f"What should you do next? (Iteration {iteration + 1})\n"
-        
+
         return prompt
 
     def _parse_autonomous_action(self, response: str) -> dict:
@@ -1624,6 +1416,18 @@ Think step-by-step internally, but output ONLY the JSON action object. No explan
                 norm = {str(k).lower(): v for k, v in data.items()}
                 action_type = norm.get("type") or norm.get("action") or "think"
                 action_type = str(action_type).lower()
+                
+                # Handle LLM putting tool name in "action" field instead of "type"
+                # e.g., {"action": "web_search", "tool": "web_search", ...}
+                if action_type not in ["tool", "delegate", "finish", "think"]:
+                    # LLM likely put the tool name in "action" field
+                    # Check if there's a "tool" field, or use "action" value as tool name
+                    if norm.get("tool") or norm.get("params"):
+                        # This looks like a tool call
+                        tool = norm.get("tool") or action_type
+                        params = norm.get("params") or {}
+                        return {"type": "tool", "tool": tool, "params": params}
+                
                 # Map common aliases
                 if action_type == "tool":
                     tool = norm.get("tool") or norm.get("tool_name")
