@@ -174,6 +174,17 @@ class EvenAgeConfig(BaseSettings):
         description="Name of the coordinator agent (used when has_coordinator=True)"
     )
     
+    # Delegation execution style
+    # If True, delegation will block and wait for responses (legacy behavior).
+    # If False, use the non-blocking pause/resume orchestration model (default).
+    blocking_delegation: bool = Field(
+        default=True,
+        description=(
+            "If True, delegation blocks and waits for delegated-agent responses. "
+            "If False, delegation uses non-blocking pause/resume (recommended for production)."
+        ),
+    )
+    
     # MCP (Model Context Protocol) Integration - DISABLED FOR THIS RELEASE
     # mcp_servers: list[dict[str, str]] = Field(
     #     default_factory=list,
