@@ -147,11 +147,19 @@ class DatabaseService:
     # Job management
 
     def create_job(self, job_id: str | None, pipeline: str, inputs: dict) -> str:
-        """Create a new job record."""
+        """Create a new job record, or reuse existing if job_id already exists (for sequential chains)."""
         if job_id is None:
             job_id = str(uuid.uuid4())
 
         with self.get_session() as session:
+            # Check if job already exists (for sequential mode where agents share job_id)
+            existing_job = session.query(Job).filter_by(job_id=job_id).first()
+            if existing_job:
+                # Job already exists (e.g., from first agent in sequential chain)
+                # Just return the job_id without creating a duplicate
+                return job_id
+            
+            # Create new job
             job = Job(
                 job_id=job_id,
                 pipeline_name=pipeline,
@@ -178,14 +186,20 @@ class DatabaseService:
             if not job:
                 return None
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                if not dt:
+                    return None
+                # stored as naive UTC; normalize to Z-suffixed ISO for clients
+                return dt.isoformat() + "Z"
+
             return {
                 "job_id": job.job_id,
                 "pipeline_name": job.pipeline_name,
                 "status": job.status,
                 "inputs": job.inputs,
                 "outputs": job.outputs,
-                "created_at": job.created_at.isoformat() if job.created_at else None,
-                "completed_at": job.completed_at.isoformat() if job.completed_at else None,
+                "created_at": _iso_z(job.created_at),
+                "completed_at": _iso_z(job.completed_at),
             }
 
     def list_jobs(self, limit: int = 50) -> list[dict]:
@@ -193,12 +207,15 @@ class DatabaseService:
         with self.get_session() as session:
             jobs = session.query(Job).order_by(Job.created_at.desc()).limit(limit).all()
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return [
                 {
                     "job_id": job.job_id,
                     "pipeline_name": job.pipeline_name,
                     "status": job.status,
-                    "created_at": job.created_at.isoformat() if job.created_at else None,
+                    "created_at": _iso_z(job.created_at),
                 }
                 for job in jobs
             ]
@@ -247,14 +264,17 @@ class DatabaseService:
             if not prompt:
                 return None
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return {
                 "prompt_id": prompt.prompt_id,
                 "prompt_name": prompt.prompt_name,
                 "status": prompt.status,
                 "inputs": prompt.inputs,
                 "outputs": prompt.outputs,
-                "created_at": prompt.created_at.isoformat() if prompt.created_at else None,
-                "completed_at": prompt.completed_at.isoformat() if prompt.completed_at else None,
+                "created_at": _iso_z(prompt.created_at),
+                "completed_at": _iso_z(prompt.completed_at),
             }
 
     def list_prompts(self, limit: int = 50) -> list[dict]:
@@ -262,12 +282,15 @@ class DatabaseService:
         with self.get_session() as session:
             prompts = session.query(PromptExecution).order_by(PromptExecution.created_at.desc()).limit(limit).all()
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return [
                 {
                     "prompt_id": prompt.prompt_id,
                     "prompt_name": prompt.prompt_name,
                     "status": prompt.status,
-                    "created_at": prompt.created_at.isoformat() if prompt.created_at else None,
+                    "created_at": _iso_z(prompt.created_at),
                 }
                 for prompt in prompts
             ]
@@ -277,6 +300,9 @@ class DatabaseService:
         with self.get_session() as session:
             traces = session.query(Trace).filter_by(job_id=job_id).order_by(Trace.timestamp).all()
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return [
                 {
                     "id": trace.id,
@@ -284,7 +310,7 @@ class DatabaseService:
                     "agent_name": trace.agent_name,
                     "event_type": trace.event_type,
                     "payload": trace.payload,
-                    "timestamp": trace.timestamp.isoformat() if trace.timestamp else None,
+                    "timestamp": _iso_z(trace.timestamp),
                 }
                 for trace in traces
             ]
@@ -312,6 +338,9 @@ class DatabaseService:
 
             traces = query.order_by(Trace.timestamp.desc()).limit(limit).all()
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return [
                 {
                     "id": trace.id,
@@ -319,7 +348,7 @@ class DatabaseService:
                     "agent_name": trace.agent_name,
                     "event_type": trace.event_type,
                     "payload": trace.payload,
-                    "timestamp": trace.timestamp.isoformat() if trace.timestamp else None,
+                    "timestamp": _iso_z(trace.timestamp),
                 }
                 for trace in traces
             ]
@@ -330,13 +359,15 @@ class DatabaseService:
             trace = session.query(Trace).filter_by(id=trace_id).first()
             if not trace:
                 return None
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
             return {
                 "id": trace.id,
                 "job_id": trace.job_id,
                 "agent_name": trace.agent_name,
                 "event_type": trace.event_type,
                 "payload": trace.payload,
-                "timestamp": trace.timestamp.isoformat() if trace.timestamp else None,
+                "timestamp": _iso_z(trace.timestamp),
             }
 
     # Memory
@@ -408,11 +439,14 @@ class DatabaseService:
         with self.get_session() as session:
             agents = session.query(AgentRegistry).all()
 
+            def _iso_z(dt: datetime | None) -> str | None:
+                return dt.isoformat() + "Z" if dt else None
+
             return [
                 {
                     "agent_name": agent.agent_name,
                     "metadata": agent.meta,
-                    "last_seen": agent.last_seen.isoformat() if agent.last_seen else None,
+                    "last_seen": _iso_z(agent.last_seen),
                 }
                 for agent in agents
             ]
@@ -454,3 +488,48 @@ class DatabaseService:
                 "total_tools_called": tool_calls,
                 "cache_hits": cache_hits,
             }
+
+    # Token usage metrics aggregated from llm_usage traces
+    def get_token_usage(self, job_id: str) -> dict:
+        """Aggregate token usage for a job from llm_usage traces."""
+        with self.get_session() as session:
+            traces = (
+                session.query(Trace)
+                .filter_by(job_id=job_id, event_type="llm_usage")
+                .all()
+            )
+            total_prompt = 0
+            total_completion = 0
+            total = 0
+            breakdown: dict[tuple[str | None, str | None], dict] = {}
+            for tr in traces:
+                payload = tr.payload or {}
+                pt = int(payload.get("prompt_tokens") or 0)
+                ct = int(payload.get("completion_tokens") or 0)
+                tt = int(payload.get("total_tokens") or (pt + ct))
+                prov = payload.get("provider")
+                model = payload.get("model")
+                total_prompt += pt
+                total_completion += ct
+                total += tt
+                key = (prov, model)
+                if key not in breakdown:
+                    breakdown[key] = {
+                        "provider": prov,
+                        "model": model,
+                        "prompt_tokens": 0,
+                        "completion_tokens": 0,
+                        "total_tokens": 0,
+                        "calls": 0,
+                    }
+                breakdown[key]["prompt_tokens"] += pt
+                breakdown[key]["completion_tokens"] += ct
+                breakdown[key]["total_tokens"] += tt
+                breakdown[key]["calls"] += 1
+
+            return {
+                "prompt_tokens": total_prompt,
+                "completion_tokens": total_completion,
+                "total_tokens": total,
+                "by_model": list(breakdown.values()),
+            }
