@@ -36,6 +36,8 @@ class AgentRunner:
         """
         Initialize runner.
         
+        
+        
         Args:
             agent: Agent instance (created if None)
             env_config: Environment configuration
@@ -74,7 +76,23 @@ class AgentRunner:
                 mod = importlib.import_module(f"agents.{agent_name}")
                 candidate = getattr(mod, agent_name, None)
                 if candidate and hasattr(candidate, "handle"):
-                    agent = candidate  # type: ignore[assignment]
+                    # If the project exposed a module-level Agent instance, re-instantiate
+                    # it with this runner's env_config to ensure runtime flags (like
+                    # blocking_delegation) are respected.
+                    try:
+                        AgentCls = candidate.__class__
+                        agent_cfg = getattr(candidate, "config", None)
+                        if agent_cfg is None:
+                            # Fallback to simple AgentConfig
+                            agent_cfg = AgentConfig(
+                                name=agent_name,
+                                role=getattr(candidate, "ROLE", "Agent"),
+                                goal=getattr(candidate, "GOAL", "Execute tasks")
+                            )
+                        agent = AgentCls(agent_cfg, self.env_config)
+                    except Exception:
+                        # Fall back to using the provided module-level instance
+                        agent = candidate  # type: ignore[assignment]
                 else:
                     raise AttributeError("No module-level agent instance found")
             except Exception:
@@ -94,8 +112,36 @@ class AgentRunner:
                         f"or legacy 'agents/{agent_name}/handler.py' defines '{agent_name.capitalize()}Agent'. Root cause: {primary_err}"
                     )
             # Ensure bus registration
+            # Ensure the agent instance uses this runner's env_config and backends.
             try:
-                await agent.connect_bus()
+                # Overwrite env_config and recreate factory/llm/cache/storage to match runner
+                setattr(agent, "env_config", self.env_config)
+                try:
+                    setattr(agent, "factory", BackendFactory(self.env_config))
+                except Exception:
+                    pass
+                try:
+                    # Recreate LLM backend from runner config so correct provider (gemini/openai) is used
+                    if hasattr(agent, "factory"):
+                        setattr(agent, "llm", agent.factory.create_llm_backend())
+                except Exception:
+                    pass
+                # Connect/register on the bus
+                try:
+                    await agent.connect_bus()
+                except Exception:
+                    pass
+            except Exception:
+                # Best-effort; continue if we can't rewire the agent
+                pass
+
+            # Honor a caller-provided override to disable delegation for this run.
+            # The API sets a private `_allow_delegation` key for single-shot prompts.
+            try:
+                if isinstance(inputs, dict) and "_allow_delegation" in inputs:
+                    override = inputs.get("_allow_delegation")
+                    if hasattr(agent, "config") and hasattr(agent.config, "allow_delegation"):
+                        agent.config.allow_delegation = bool(override)
             except Exception:
                 pass
         elif self.agent:
@@ -135,36 +181,6 @@ class AgentRunner:
 
             # Handle task
             result = await agent.handle(task)
-            
-            # Check if execution was PAUSED (waiting for delegation)
-            if result.get("status") == "paused":
-                delegation_task_id = result.get("delegation_task_id")
-                delegated_agent = result.get("delegated_agent")
-                
-                # Store paused state in job metadata
-                self.database.save_result(
-                    job_id=job_id,
-                    outputs={
-                        "status": "paused",
-                        "delegation_task_id": delegation_task_id,
-                        "delegated_agent": delegated_agent,
-                        "message": result.get("message", "Execution paused")
-                    },
-                    status="paused"
-                )
-                
-                # Set up delegation response listener
-                # When delegation completes, orchestrator should call handle_delegation_response()
-                duration_ms = int((time.time() - start) * 1000)
-                return {
-                    "job_id": job_id,
-                    "status": "paused",
-                    "delegation_task_id": delegation_task_id,
-                    "delegated_agent": delegated_agent,
-                    "message": f"Execution paused, waiting for {delegated_agent}",
-                    "duration_ms": duration_ms,
-                    "agent": pipeline_name
-                }
 
             # Save result
             outputs = result.get("result", {})
@@ -214,159 +230,6 @@ class AgentRunner:
                 with suppress(Exception):
                     self._stop_inline_workers()
     
-    async def handle_delegation_response(
-        self,
-        delegation_task_id: str,
-        delegation_result: dict
-    ) -> dict:
-        """
-        Handle a completed delegation and resume parent agent execution.
-        
-        This is called by the orchestrator when a delegated task completes.
-        It loads the parent agent, resumes from saved context, and continues execution.
-        
-        Args:
-            delegation_task_id: The task_id of the completed delegation
-            delegation_result: The result returned by the delegated agent
-        
-        Returns:
-            Result from resumed execution (may be paused again if chained delegations)
-        
-        Example orchestrator pattern:
-            ```python
-            # 1. Run parent agent
-            result = await runner.run(inputs, agent_name="coordinator")
-            
-            # 2. If paused, wait for delegation to complete
-            if result["status"] == "paused":
-                delegation_task_id = result["delegation_task_id"]
-                
-                # Listen for delegation completion (via message bus or polling)
-                delegation_result = await wait_for_delegation(delegation_task_id)
-                
-                # 3. Resume parent execution
-                final_result = await runner.handle_delegation_response(
-                    delegation_task_id,
-                    delegation_result
-                )
-            ```
-        """
-        import logging
-        logger = logging.getLogger(__name__)
-        
-        # Load the parent agent from paused context
-        # We need to find which agent was paused - check cache/memory
-        bus = self.factory.create_queue_backend()
-        cache = self.factory.create_cache_backend()
-        
-        # Try to load context from cache to identify parent agent
-        cache_key = f"paused_context:{delegation_task_id}"
-        context_data = None
-        
-        try:
-            if hasattr(cache, 'get') and asyncio.iscoroutinefunction(cache.get):
-                context_data = await cache.get(cache_key)
-            else:
-                context_data = cache.get(cache_key)
-        except Exception as e:
-            logger.warning(f"Failed to load context from cache: {e}")
-        
-        if not context_data:
-            return {
-                "status": "error",
-                "error": f"Cannot resume: paused context not found for delegation {delegation_task_id}"
-            }
-        
-        parent_agent_name = context_data.get("parent_agent_name")
-        parent_task_id = context_data.get("parent_task_id")
-        
-        logger.info(
-            f"Resuming agent '{parent_agent_name}' from delegation_task_id={delegation_task_id}"
-        )
-        
-        # Load the parent agent
-        try:
-            mod = importlib.import_module(f"agents.{parent_agent_name}")
-            agent = getattr(mod, parent_agent_name, None)
-            if not agent or not hasattr(agent, "resume_paused_execution"):
-                raise AttributeError(f"Agent {parent_agent_name} not found or doesn't support resumption")
-        except Exception as e:
-            logger.error(f"Failed to load parent agent '{parent_agent_name}': {e}")
-            return {
-                "status": "error",
-                "error": f"Failed to load parent agent: {e}"
-            }
-        
-        # Ensure bus connection
-        try:
-            await agent.connect_bus()
-        except Exception:
-            pass
-        
-        # Resume execution
-        start = time.time()
-        try:
-            result = await agent.resume_paused_execution(
-                delegation_task_id,
-                delegation_result
-            )
-            
-            # Check if paused again (chained delegations)
-            if result.get("status") == "paused":
-                logger.info(
-                    f"Agent '{parent_agent_name}' paused again at delegation_task_id={result.get('delegation_task_id')}"
-                )
-                # Update job with new paused state
-                self.database.save_result(
-                    job_id=parent_task_id,
-                    outputs={
-                        "status": "paused",
-                        "delegation_task_id": result.get("delegation_task_id"),
-                        "delegated_agent": result.get("delegated_agent"),
-                        "message": result.get("message", "Execution paused again")
-                    },
-                    status="paused"
-                )
-                return result
-            
-            # Execution completed
-            outputs = result.get("result", {})
-            status = result.get("status", "completed")
-            error = result.get("error")
-            
-            if error:
-                outputs = {"error": error, **outputs}
-            
-            self.database.save_result(
-                job_id=parent_task_id,
-                outputs=outputs,
-                status=status
-            )
-            
-            duration_ms = int((time.time() - start) * 1000)
-            return {
-                "job_id": parent_task_id,
-                "status": status,
-                "result": outputs,
-                "error": error,
-                "duration_ms": duration_ms,
-                "agent": parent_agent_name
-            }
-            
-        except Exception as e:
-            logger.exception(f"Error resuming agent '{parent_agent_name}': {e}")
-            self.database.save_result(
-                job_id=parent_task_id,
-                outputs={"error": str(e)},
-                status="failed"
-            )
-            return {
-                "job_id": parent_task_id,
-                "status": "failed",
-                "error": str(e),
-                "agent": parent_agent_name
-            }
-
     def _discover_local_agents(self, exclude: list[str] | None = None) -> list[str]:
         """Discover local agents under the 'agents' package for inline execution."""
         names: list[str] = []
