@@ -35,10 +35,12 @@ class TaskDelegationTool:
     async def delegate_task(
         self,
         agent_name: str,
-        task_description: str,
+        task_description: str | None = None,
+        task: str | None = None,
         task_data: dict | None = None,
         wait_for_response: bool = False,
-        timeout_seconds: int = 60
+        timeout_seconds: int = 60,
+        **kwargs
     ) -> dict:
         """
         Delegate a task to another agent for execution.
@@ -64,11 +66,14 @@ class TaskDelegationTool:
         - You do NOT need to call retrieve_artifact unless response explicitly says {"stored_in_registry": true}
         - task_id is for tracking only, NOT for artifact retrieval
         """
+        # Backwards-compatible param mapping: accept either `task_description` or
+        # `task` (some LLMs produce `task`), and normalize to payload.description
         task_data = task_data or {}
+        desc = task_description or task or kwargs.get("task_description") or kwargs.get("description") or ""
 
         # Build task payload
         payload = {
-            "description": task_description,
+            "description": desc,
             "data": task_data,
             "delegated_at": datetime.utcnow().isoformat()
         }
@@ -261,8 +266,11 @@ class ArtifactStorageTool:
             Dict with artifact data, metadata, and retrieval info
         """
         # New path: direct bucket/key retrieval (e.g., offloaded responses)
+        # NOTE: artifact storage uses the bucket "evenage-artifacts" by default
+        # for stored task artifacts. Use that as the default to avoid confusing
+        # callers that pass only a key or a job id.
         if key is not None:
-            use_bucket = bucket or "evenage"
+            use_bucket = bucket or "evenage-artifacts"
             try:
                 data = await self.storage.get_object(bucket=use_bucket, key=key)
                 try:
